@@ -14,21 +14,30 @@
 
 import requests
 
-from google.api_core import exceptions
+from google.api_core import exceptions as api_exceptions
 from google.api_core import retry
+from google.auth import exceptions as auth_exceptions
 
 import json
 
 
-_RETRYABLE_TYPES = (
-    exceptions.TooManyRequests,  # 429
-    exceptions.InternalServerError,  # 500
-    exceptions.BadGateway,  # 502
-    exceptions.ServiceUnavailable,  # 503
-    exceptions.GatewayTimeout,  # 504
+# ConnectionError is a built-in exception only in Python3 and not in Python2.
+try:
+    _RETRYABLE_STDLIB_TYPES = (ConnectionError,)
+except NameError:
+    _RETRYABLE_STDLIB_TYPES = ()
+
+
+_RETRYABLE_TYPES = _RETRYABLE_STDLIB_TYPES + (
+    api_exceptions.TooManyRequests,  # 429
+    api_exceptions.InternalServerError,  # 500
+    api_exceptions.BadGateway,  # 502
+    api_exceptions.ServiceUnavailable,  # 503
+    api_exceptions.GatewayTimeout,  # 504
     requests.ConnectionError,
 )
 
+
 # Some retriable errors don't have their own custom exception in api_core.
 _ADDITIONAL_RETRYABLE_STATUS_CODES = (408,)
 
@@ -37,8 +46,10 @@ def _should_retry(exc):
     """Predicate for determining when to retry."""
     if isinstance(exc, _RETRYABLE_TYPES):
         return True
-    elif isinstance(exc, exceptions.GoogleAPICallError):
+    elif isinstance(exc, api_exceptions.GoogleAPICallError):
         return exc.code in _ADDITIONAL_RETRYABLE_STATUS_CODES
+    elif isinstance(exc, auth_exceptions.TransportError):
+        return _should_retry(exc.args[0])
     else:
         return False
 
@@ -99,13 +110,13 @@ class ConditionalRetryPolicy(object):
 def is_generation_specified(query_params):
     """Return True if generation or if_generation_match is specified."""
     generation = query_params.get("generation") is not None
-    if_generation_match = query_params.get("if_generation_match") is not None
+    if_generation_match = query_params.get("ifGenerationMatch") is not None
     return generation or if_generation_match
 
 
 def is_metageneration_specified(query_params):
     """Return True if if_metageneration_match is specified."""
-    if_metageneration_match = query_params.get("if_metageneration_match") is not None
+    if_metageneration_match = query_params.get("ifMetagenerationMatch") is not None
     return if_metageneration_match
 
 
@@ -128,9 +139,29 @@ def is_etag_in_json(data):
 DEFAULT_RETRY_IF_GENERATION_SPECIFIED = ConditionalRetryPolicy(
     DEFAULT_RETRY, is_generation_specified, ["query_params"]
 )
+"""Conditional wrapper for the default retry object.
+
+This retry setting will retry all _RETRYABLE_TYPES and any status codes from
+_ADDITIONAL_RETRYABLE_STATUS_CODES, but only if the request included an
+``ifGenerationMatch`` header.
+"""
+
 DEFAULT_RETRY_IF_METAGENERATION_SPECIFIED = ConditionalRetryPolicy(
     DEFAULT_RETRY, is_metageneration_specified, ["query_params"]
 )
+"""Conditional wrapper for the default retry object.
+
+This retry setting will retry all _RETRYABLE_TYPES and any status codes from
+_ADDITIONAL_RETRYABLE_STATUS_CODES, but only if the request included an
+``ifMetagenerationMatch`` header.
+"""
+
 DEFAULT_RETRY_IF_ETAG_IN_JSON = ConditionalRetryPolicy(
     DEFAULT_RETRY, is_etag_in_json, ["data"]
 )
+"""Conditional wrapper for the default retry object.
+
+This retry setting will retry all _RETRYABLE_TYPES and any status codes from
+_ADDITIONAL_RETRYABLE_STATUS_CODES, but only if the request included an
+``ETAG`` entry in its payload.
+"""
